package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.IBoard;
import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.player.IAI;
import inf101.v18.connectfour.player.IPlayer;

import java.awt.*;
import java.util.InputMismatchException;
import java.util.Scanner;

/**
 * An abstract class implementing methods common for all CLI interfaces.
 */
public abstract class CLI implements IGameInterface {
    private final String ANSI_RESET = "\u001B[0m";
    private final String ANSI_GREEN = "\u001B[92m";
    private final String ANSI_RED = "\u001B[91m";
    private final String ANSI_BLUE = "\u001B[94m";
    private final String ANSI_YELLOW = "\u001B[93m";
    private final String ANSI_MAGENTA = "\u001B[95m";
    private final String ANSI_CYAN = "\u001B[96m";

    IBoard board;
    IPlayer currentPlayer;
    private final Scanner in = new Scanner(System.in);

    /**
     * Converts a Color to an ansi colored string because CLI doesn't support RGB.
     *
     * @param color The color to convert
     * @return      An ansi colored O or an empty string
     */
    String colorToAnsi(Color color, char symbol) {
        if (color == Color.green) {
            return ANSI_GREEN + symbol + ANSI_RESET;
        } else if (color == Color.red) {
            return ANSI_RED + symbol + ANSI_RESET;
        } else if (color == Color.blue) {
            return ANSI_BLUE + symbol + ANSI_RESET;
        } else if (color == Color.yellow) {
            return ANSI_YELLOW + symbol + ANSI_RESET;
        } else if (color == Color.magenta) {
            return ANSI_MAGENTA + symbol + ANSI_RESET;
        } else if (color == Color.cyan) {
            return ANSI_CYAN + symbol + ANSI_RESET;
        } else {
            return ".";
        }
    }

    /**
     * Reads a name from the commandline.
     *
     * @param i An index representing the user being named
     * @return  A non-empty name
     */
    String readName(int i) {
        String name;
        do {
            displayMessage("Name of player " + i + ":");
            name = in.nextLine();
        } while (name.equals(""));
        return name;
    }

    /**
     * Asks the user if they want to play alone against an AI, or with a friend.
     *
     * @return True if the player's input contains y
     */
    boolean playingAlone() {
        displayMessage("Will you be playing alone? (y/n):");
        return in.nextLine().toLowerCase().contains("y");
    }

    /**
     * Gets the correct symbol for a player color
     * @param color <p>A color</p>
     * @return <p>A symbol</p>
     */
    char getSymbol(Color color) {
        if (color == Color.red) {
            return 'X';
        } else if (color == Color.green) {
            return 'O';
        }
        return 'O';
    }

    @Override
    public void doTurn() {
        boolean placed = false;
        System.out.printf("%nIt's %s's turn (%s)%n", currentPlayer.getName(), colorToAnsi(currentPlayer.getColor(),
                getSymbol(currentPlayer.getColor())));
        if (currentPlayer instanceof IAI) {
            if (board instanceof IVerticalBoard) {
                placed = ((IVerticalGameInterface) this).dropObject(((IAI)currentPlayer).getColumn((IVerticalBoard) board));
            } else {
                int[] choice = ((IAI) currentPlayer).getXY(board);
                try {
                    placed = placePlayerObject(choice[0], choice[1]);
                } catch (IllegalAccessException e) {
                    e.printStackTrace(); //If this happens, the AI is broken.
                }
            }
        } else {
            if (board instanceof IVerticalBoard) {
                displayMessage("Drop token in (i):");
                try {
                    int i = in.nextInt();
                    placed = ((IVerticalGameInterface)this).dropObject(i - 1);
                } catch (InputMismatchException | IllegalArgumentException ignored) {
                } finally {
                    in.nextLine();
                }
            } else {
                displayMessage("Place token at (x,y):");
                try {
                    String s = in.nextLine();
                    if (s.contains(",")) {
                        String[] numbers = s.split(",");
                        placed = placePlayerObject(Integer.parseInt(numbers[0]) - 1, Integer.parseInt(numbers[1]) - 1);
                    }
                } catch (InputMismatchException | IllegalArgumentException | IllegalAccessException ignored) {}
            }
        }
        if (!placed) {
            displayMessage("Invalid input. Please try again.");
        }
    }

    @Override
    public boolean hasDrawn() {
        return board.isFull();
    }

    @Override
    public boolean hasWon() {
        return board.won();
    }

    @Override
    public void displayMessage(String s) {
        System.out.println(s);
    }
}
